@@ -9,7 +9,13 @@ import numpy as np
 from sklearn.utils import check_random_state, check_array
 from sklearn.preprocessing import normalize
 from sklearn.base import BaseEstimator, TransformerMixin
-from scipy.sparse import csr_matrix, coo_matrix, isspmatrix_csr, vstack as sparse_vstack, issparse
+from scipy.sparse import (
+    csr_matrix,
+    coo_matrix,
+    isspmatrix_csr,
+    vstack as sparse_vstack,
+    issparse,
+)
 
 import heapq
 
@@ -31,6 +37,7 @@ from pynndescent.utils import (
     apply_graph_updates_high_memory,
     apply_graph_updates_low_memory,
     initalize_heap_from_graph_indices,
+    initalize_heap_from_graph_indices_and_distances,
     sparse_initalize_heap_from_graph_indices,
 )
 
@@ -588,6 +595,16 @@ class NNDescent:
     tree_init: bool (optional, default=True)
         Whether to use random projection trees for initialization.
 
+    init_graph: np.ndarray (optional, default=None)
+        2D array of indices of candidate neighbours of the shape
+        (data.shape[0], n_neighbours). If the j-th neighbour of the i-th
+        instances is unknown, use init_graph[i, j] = -1
+
+    init_dist: np.ndarray (optional, default=None)
+        2D array with the same shape as init_graph,
+        such that metric(data[i], data[init_graph[i, j]]) equals
+        init_dist[i, j]
+
     random_state: int, RandomState instance or None, optional (default: None)
         If int, random_state is the seed used by the random number generator;
         If RandomState instance, random_state is the random number generator;
@@ -658,6 +675,7 @@ class NNDescent:
         n_search_trees=1,
         tree_init=True,
         init_graph=None,
+        init_dist=None,
         random_state=None,
         low_memory=True,
         max_candidates=None,
@@ -676,6 +694,7 @@ class NNDescent:
             n_iters = max(5, int(round(np.log2(data.shape[0]))))
 
         self.n_trees = n_trees
+        self.n_trees_after_update = max(1, int(np.round(self.n_trees / 3)))
         self.n_neighbors = n_neighbors
         self.metric = metric
         self.metric_kwds = metric_kwds
@@ -693,7 +712,9 @@ class NNDescent:
         self.parallel_batch_queries = parallel_batch_queries
         self.verbose = verbose
 
-        if getattr(data, "dtype", None) == np.float32 and (issparse(data) or is_c_contiguous(data)):
+        if getattr(data, "dtype", None) == np.float32 and (
+            issparse(data) or is_c_contiguous(data)
+        ):
             copy_on_normalize = True
         else:
             copy_on_normalize = False
@@ -880,9 +901,18 @@ class NNDescent:
                 if init_graph.shape[0] != self._raw_data.shape[0]:
                     raise ValueError("Init graph size does not match dataset size!")
                 _init_graph = make_heap(init_graph.shape[0], self.n_neighbors)
-                _init_graph = initalize_heap_from_graph_indices(
-                    _init_graph, init_graph, data, self._distance_func
-                )
+                if init_dist is None:
+                    _init_graph = initalize_heap_from_graph_indices(
+                        _init_graph, init_graph, data, self._distance_func
+                    )
+                elif init_graph.shape != init_dist.shape:
+                    raise ValueError(
+                        "The shapes of init graph and init distances do not match!"
+                    )
+                else:
+                    _init_graph = initalize_heap_from_graph_indices_and_distances(
+                        _init_graph, init_graph, init_dist
+                    )
 
             if verbose:
                 print(ts(), "NN descent for", str(n_iters), "iterations")
@@ -905,8 +935,8 @@ class NNDescent:
         if np.any(self._neighbor_graph[0] < 0):
             warn(
                 "Failed to correctly find n_neighbors for some samples."
-                "Results may be less than ideal. Try re-running with"
-                "different parameters."
+                " Results may be less than ideal. Try re-running with"
+                " different parameters."
             )
 
         numba.set_num_threads(self._original_num_threads)
@@ -960,7 +990,9 @@ class NNDescent:
                         self._angular_trees,
                     )
                     self._search_forest = [
-                        convert_tree_format(tree, self._raw_data.shape[0])
+                        convert_tree_format(
+                            tree, self._raw_data.shape[0], self._raw_data.shape[1]
+                        )
                         for tree in rp_forest
                     ]
                 else:
@@ -979,7 +1011,9 @@ class NNDescent:
                 best_trees = [self._rp_forest[idx] for idx in best_tree_indices]
                 del self._rp_forest
                 self._search_forest = [
-                    convert_tree_format(tree, self._raw_data.shape[0])
+                    convert_tree_format(
+                        tree, self._raw_data.shape[0], self._raw_data.shape[1]
+                    )
                     for tree in best_trees
                 ]
 
@@ -1132,7 +1166,9 @@ class NNDescent:
             if self._is_sparse:
                 self._raw_data = self._raw_data[self._vertex_order, :]
             else:
-                self._raw_data = np.ascontiguousarray(self._raw_data[self._vertex_order, :])
+                self._raw_data = np.ascontiguousarray(
+                    self._raw_data[self._vertex_order, :]
+                )
 
             tree_order = np.argsort(self._vertex_order)
             self._search_forest = tuple(
@@ -1186,6 +1222,7 @@ class NNDescent:
 
             self._tree_search = tree_search_closure
         else:
+
             @numba.njit()
             def tree_search_closure(point, rng_state):
                 return (0, 0)
@@ -1316,9 +1353,12 @@ class NNDescent:
             return result
 
         self._search_function = search_closure
-        self._deheap_function = numba.njit(parallel=self.parallel_batch_queries)(
-            deheap_sort.py_func
-        )
+        if hasattr(deheap_sort, "py_func"):
+            self._deheap_function = numba.njit(parallel=self.parallel_batch_queries)(
+                deheap_sort.py_func
+            )
+        else:
+            self._deheap_function = deheap_sort
 
         # Force compilation of the search function (hardcoded k, epsilon)
         query_data = self._raw_data[:1]
@@ -1367,6 +1407,7 @@ class NNDescent:
 
             self._tree_search = sparse_tree_search_closure
         else:
+
             @numba.njit()
             def sparse_tree_search_closure(point_inds, point_data, rng_state):
                 return (0, 0)
@@ -1426,7 +1467,7 @@ class NNDescent:
                 current_query_data = query_data[query_indptr[i] : query_indptr[i + 1]]
 
                 if dist == alternative_dot or dist == alternative_cosine:
-                    norm = np.sqrt((current_query_data ** 2).sum())
+                    norm = np.sqrt((current_query_data**2).sum())
                     if norm > 0.0:
                         current_query_data = current_query_data / norm
                     else:
@@ -1456,9 +1497,11 @@ class NNDescent:
                         data_indptr[candidate] : data_indptr[candidate + 1]
                     ]
 
-                    d = np.float32(dist(
-                        from_inds, from_data, current_query_inds, current_query_data
-                    ))
+                    d = np.float32(
+                        dist(
+                            from_inds, from_data, current_query_inds, current_query_data
+                        )
+                    )
                     # indices are guaranteed different
                     simple_heap_push(heap_priorities, heap_indices, d, candidate)
                     heapq.heappush(seed_set, (d, candidate))
@@ -1477,12 +1520,14 @@ class NNDescent:
                                 data_indptr[candidate] : data_indptr[candidate + 1]
                             ]
 
-                            d = np.float32(dist(
-                                from_inds,
-                                from_data,
-                                current_query_inds,
-                                current_query_data,
-                            ))
+                            d = np.float32(
+                                dist(
+                                    from_inds,
+                                    from_data,
+                                    current_query_inds,
+                                    current_query_data,
+                                )
+                            )
 
                             simple_heap_push(
                                 heap_priorities, heap_indices, d, candidate
@@ -1512,12 +1557,14 @@ class NNDescent:
                                 data_indptr[candidate] : data_indptr[candidate + 1]
                             ]
 
-                            d = np.float32(dist(
-                                from_inds,
-                                from_data,
-                                current_query_inds,
-                                current_query_data,
-                            ))
+                            d = np.float32(
+                                dist(
+                                    from_inds,
+                                    from_data,
+                                    current_query_inds,
+                                    current_query_data,
+                                )
+                            )
 
                             if d < distance_bound:
                                 simple_heap_push(
@@ -1536,9 +1583,12 @@ class NNDescent:
             return result
 
         self._search_function = search_closure
-        self._deheap_function = numba.njit(parallel=self.parallel_batch_queries)(
-            deheap_sort.py_func
-        )
+        if hasattr(deheap_sort, "py_func"):
+            self._deheap_function = numba.njit(parallel=self.parallel_batch_queries)(
+                deheap_sort.py_func
+            )
+        else:
+            self._deheap_function = deheap_sort
 
         # Force compilation of the search function (hardcoded k, epsilon)
         query_data = self._raw_data[:1]
@@ -1665,29 +1715,120 @@ class NNDescent:
 
         return indices, dists
 
-    def update(self, X):
+    def update(self, xs_fresh=None, xs_updated=None, updated_indices=None):
+        """
+        Updates the index with a) fresh data (that is appended to
+        the existing data), and b) data that was only updated (but should not be appended
+        to the existing data).
+
+        Not applicable to sparse data yet.
+
+        Parameters
+        ----------
+        xs_fresh: np.ndarray (optional, default=None)
+            2D array of the shape (n_fresh, dim) where dim is the dimension
+            of the data from which we built self.
+
+        xs_updated: np.ndarray (optional, default=None)
+            2D array of the shape (n_updates, dim) where dim is the dimension
+            of the data from which we built self.
+
+        updated_indices: array-like of size n_updates (optional, default=None)
+            Something that is convertable to list of ints.
+            If self is currently built from xs, then xs[update_indices[i]]
+            will be replaced by xs_updated[i].
+
+        Returns
+        -------
+            None
+        """
         current_random_state = check_random_state(self.random_state)
         rng_state = current_random_state.randint(INT32_MIN, INT32_MAX, 3).astype(
             np.int64
         )
-        X = check_array(X, dtype=np.float32, accept_sparse="csr", order="C")
-
+        error_sparse_to_do = NotImplementedError("Sparse update not complete yet")
+        # input checks
+        if xs_updated is not None:
+            xs_updated = check_array(
+                xs_updated, dtype=np.float32, accept_sparse="csr", order="C"
+            )
+            if updated_indices is None:
+                raise ValueError(
+                    "If xs_updated are provided, updated_indices must also be provided!"
+                )
+            if self._is_sparse:
+                raise error_sparse_to_do
+            else:
+                try:
+                    updated_indices = list(map(int, updated_indices))
+                except (TypeError, ValueError):
+                    raise ValueError(
+                        "Could not convert updated indices to list of int(s)."
+                    )
+                n1 = len(updated_indices)
+                n2 = xs_updated.shape[0]
+                if n1 != n2:
+                    raise ValueError(
+                        f"Number of updated indices ({n1}) must match "
+                        f"number of rows of xs_updated ({n2})."
+                    )
+        else:
+            if updated_indices is not None:
+                warn(
+                    "xs_updated not provided, while update_indices provided. "
+                    "They will be ignored."
+                )
+                updated_indices = None
+        if updated_indices is None:
+            # make an empty iterable instead
+            xs_updated = []
+            updated_indices = []
+        if xs_fresh is None:
+            if self._is_sparse:
+                xs_fresh = csr_matrix(
+                    ([], [], []), shape=(0, self._raw_data.shape[1]), dtype=np.float32
+                )
+            else:
+                xs_fresh = np.zeros((0, self._raw_data.shape[1]), dtype=np.float32)
+        else:
+            xs_fresh = check_array(
+                xs_fresh, dtype=np.float32, accept_sparse="csr", order="C"
+            )
+        # data preparation
         if hasattr(self, "_vertex_order"):
             original_order = np.argsort(self._vertex_order)
         else:
             original_order = np.ones(self._raw_data.shape[0], dtype=np.bool_)
-
         if self._is_sparse:
-            self._raw_data = sparse_vstack([self._raw_data, X])
+            self._raw_data = sparse_vstack([self._raw_data, xs_fresh])
+            if updated_indices:
+                # cannot be reached due to the check above,
+                # but will leave this here as a marker
+                raise error_sparse_to_do
         else:
-            self._raw_data = np.ascontiguousarray(
-                np.vstack([self._raw_data[original_order, :], X])
-            )
-
+            self._raw_data = self._raw_data[original_order, :]
+            for x_updated, i_fresh in zip(xs_updated, updated_indices):
+                self._raw_data[i_fresh] = x_updated
+            self._raw_data = np.ascontiguousarray(np.vstack([self._raw_data, xs_fresh]))
+            ns, ds = self._neighbor_graph
+            n_examples, n_neighbors = ns.shape
+            indices_set = set(updated_indices)  # for fast "is element" checks
+            for i in range(n_examples):
+                # maybe update whole row
+                if i in indices_set:
+                    ns[i] = -1
+                    ds[i] = np.inf
+                    continue
+                # maybe update some columns
+                for j in range(n_neighbors):
+                    if ns[i, j] in indices_set:
+                        ns[i, j] = -1
+                        ds[i, j] = np.inf
+        # update neighbors
         if self._is_sparse:
-            raise NotImplementedError("Sparse update not complete yet")
+            raise error_sparse_to_do
         else:
-            self.n_trees = int(np.round(self.n_trees / 3))
+            self.n_trees = self.n_trees_after_update
             self._rp_forest = make_forest(
                 self._raw_data,
                 self.n_neighbors,
@@ -1728,9 +1869,9 @@ class NNDescent:
             # Remove search graph and search function
             # and rerun prepare if it was run previously
             if (
-                    hasattr(self, "_search_graph") or
-                    hasattr(self, "_search_function") or
-                    hasattr(self, "_search_forest")
+                hasattr(self, "_search_graph")
+                or hasattr(self, "_search_function")
+                or hasattr(self, "_search_forest")
             ):
                 if hasattr(self, "_search_graph"):
                     del self._search_graph
