@@ -8,7 +8,13 @@ import numpy as np
 import numba
 import scipy.sparse
 
-from pynndescent.sparse import sparse_mul, sparse_diff, sparse_sum, arr_intersect, sparse_dot_product
+from pynndescent.sparse import (
+    sparse_mul,
+    sparse_diff,
+    sparse_sum,
+    arr_intersect,
+    sparse_dot_product,
+)
 from pynndescent.utils import tau_rand_int, norm
 import joblib
 
@@ -908,7 +914,9 @@ def sparse_select_side(hyperplane, offset, point_inds, point_data, rng_state):
     hyperplane_inds = hyperplane[0, :hyperplane_size].astype(np.int32)
     hyperplane_data = hyperplane[1, :hyperplane_size]
 
-    margin += sparse_dot_product(hyperplane_inds, hyperplane_data, point_inds, point_data)
+    margin += sparse_dot_product(
+        hyperplane_inds, hyperplane_data, point_inds, point_data
+    )
 
     if abs(margin) < EPS:
         side = tau_rand_int(rng_state) % 2
@@ -1131,36 +1139,17 @@ def num_nodes_and_leaves(tree):
     return n_nodes, n_leaves
 
 
-@numba.njit(cache=True)
-def dense_hyperplane_dim(hyperplanes):
-    for i in range(len(hyperplanes)):
-        if hyperplanes[i].shape[0] > 1:
-            return hyperplanes[i].shape[0]
-
-    raise ValueError("No hyperplanes of adequate size were found!")
-
-
-@numba.njit(cache=True)
-def sparse_hyperplane_dim(hyperplanes):
-    max_dim = 0
-    for i in range(len(hyperplanes)):
-        if hyperplanes[i].shape[1] > max_dim:
-            max_dim = hyperplanes[i].shape[1]
-    return max_dim
-
-
-def convert_tree_format(tree, data_size):
-
+def convert_tree_format(tree, data_size, data_dim):
     n_nodes, n_leaves = num_nodes_and_leaves(tree)
     is_sparse = False
     if tree.hyperplanes[0].ndim == 1:
         # dense hyperplanes
-        hyperplane_dim = dense_hyperplane_dim(tree.hyperplanes)
+        hyperplane_dim = data_dim
         hyperplanes = np.zeros((n_nodes, hyperplane_dim), dtype=np.float32)
     else:
         # sparse hyperplanes
         is_sparse = True
-        hyperplane_dim = sparse_hyperplane_dim(tree.hyperplanes)
+        hyperplane_dim = data_dim
         hyperplanes = np.zeros((n_nodes, 2, hyperplane_dim), dtype=np.float32)
         hyperplanes[:, 0, :] = -1
 
