@@ -28,9 +28,9 @@ def test_nn_descent_neighbor_accuracy(nn_data, seed):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (nn_data.shape[0] * 10)
-    assert percent_correct >= 0.98, (
-        "NN-descent did not get 99% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.98
+    ), "NN-descent did not get 99% accuracy on nearest neighbors"
 
 
 def test_angular_nn_descent_neighbor_accuracy(nn_data, seed):
@@ -47,9 +47,9 @@ def test_angular_nn_descent_neighbor_accuracy(nn_data, seed):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (nn_data.shape[0] * 10)
-    assert percent_correct >= 0.98, (
-        "NN-descent did not get 99% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.98
+    ), "NN-descent did not get 99% accuracy on nearest neighbors"
 
 
 @pytest.mark.skipif(
@@ -69,9 +69,9 @@ def test_sparse_nn_descent_neighbor_accuracy(sparse_nn_data, seed):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (sparse_nn_data.shape[0] * 10)
-    assert percent_correct >= 0.85, (
-        "Sparse NN-descent did not get 95% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.85
+    ), "Sparse NN-descent did not get 95% accuracy on nearest neighbors"
 
 
 @pytest.mark.skipif(
@@ -92,9 +92,9 @@ def test_sparse_angular_nn_descent_neighbor_accuracy(sparse_nn_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (sparse_nn_data.shape[0] * 10)
-    assert percent_correct >= 0.85, (
-        "Sparse angular NN-descent did not get 98% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.85
+    ), "Sparse angular NN-descent did not get 98% accuracy on nearest neighbors"
 
 
 def test_nn_descent_query_accuracy(nn_data):
@@ -109,9 +109,9 @@ def test_nn_descent_query_accuracy(nn_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (true_indices.shape[0] * 10)
-    assert percent_correct >= 0.95, (
-        "NN-descent query did not get 95% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.95
+    ), "NN-descent query did not get 95% accuracy on nearest neighbors"
 
 
 def test_nn_descent_query_accuracy_angular(nn_data):
@@ -126,9 +126,9 @@ def test_nn_descent_query_accuracy_angular(nn_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (true_indices.shape[0] * 10)
-    assert percent_correct >= 0.95, (
-        "NN-descent query did not get 95% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.95
+    ), "NN-descent query did not get 95% accuracy on nearest neighbors"
 
 
 def test_sparse_nn_descent_query_accuracy(sparse_nn_data):
@@ -145,9 +145,9 @@ def test_sparse_nn_descent_query_accuracy(sparse_nn_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (true_indices.shape[0] * 10)
-    assert percent_correct >= 0.95, (
-        "Sparse NN-descent query did not get 95% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.95
+    ), "Sparse NN-descent query did not get 95% accuracy on nearest neighbors"
 
 
 def test_sparse_nn_descent_query_accuracy_angular(sparse_nn_data):
@@ -164,9 +164,9 @@ def test_sparse_nn_descent_query_accuracy_angular(sparse_nn_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (true_indices.shape[0] * 10)
-    assert percent_correct >= 0.95, (
-        "Sparse NN-descent query did not get 95% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.95
+    ), "Sparse NN-descent query did not get 95% accuracy on nearest neighbors"
 
 
 def test_transformer_equivalence(nn_data):
@@ -209,9 +209,9 @@ def test_random_state_none(nn_data, spatial_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     percent_correct = num_correct / (spatial_data.shape[0] * 10)
-    assert percent_correct >= 0.99, (
-        "NN-descent did not get 99% " "accuracy on nearest neighbors"
-    )
+    assert (
+        percent_correct >= 0.99
+    ), "NN-descent did not get 99% accuracy on nearest neighbors"
 
 
 def test_deterministic():
@@ -282,9 +282,9 @@ def test_deduplicated_data_behaves_normally(seed, cosine_hang_data):
         num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
 
     proportion_correct = num_correct / (data.shape[0] * n_neighbors)
-    assert proportion_correct >= 0.95, (
-        "NN-descent did not get 95%" " accuracy on nearest neighbors"
-    )
+    assert (
+        proportion_correct >= 0.95
+    ), "NN-descent did not get 95% accuracy on nearest neighbors"
 
 
 def test_output_when_verbose_is_true(spatial_data, seed):
@@ -436,10 +436,11 @@ def test_joblib_dump():
     np.testing.assert_equal(neighbors1, neighbors2)
     np.testing.assert_equal(distances1, distances2)
 
+
 @pytest.mark.parametrize("metric", ["euclidean", "cosine"])
 def test_update_no_prepare_query_accuracy(nn_data, metric):
     nnd = NNDescent(nn_data[200:800], metric=metric, n_neighbors=10, random_state=None)
-    nnd.update(nn_data[800:])
+    nnd.update(xs_fresh=nn_data[800:])
 
     knn_indices, _ = nnd.query(nn_data[:200], k=10, epsilon=0.2)
 
@@ -455,12 +456,19 @@ def test_update_no_prepare_query_accuracy(nn_data, metric):
         "NN-descent query did not get 95% " "accuracy on nearest neighbors"
     )
 
+
 @pytest.mark.parametrize("metric", ["euclidean", "cosine"])
 def test_update_w_prepare_query_accuracy(nn_data, metric):
-    nnd = NNDescent(nn_data[200:800], metric=metric, n_neighbors=10, random_state=None, compressed=False)
+    nnd = NNDescent(
+        nn_data[200:800],
+        metric=metric,
+        n_neighbors=10,
+        random_state=None,
+        compressed=False,
+    )
     nnd.prepare()
 
-    nnd.update(nn_data[800:])
+    nnd.update(xs_fresh=nn_data[800:])
     nnd.prepare()
 
     knn_indices, _ = nnd.query(nn_data[:200], k=10, epsilon=0.2)
@@ -477,6 +485,101 @@ def test_update_w_prepare_query_accuracy(nn_data, metric):
         "NN-descent query did not get 95% " "accuracy on nearest neighbors"
     )
 
+
+@pytest.mark.parametrize("metric", ["euclidean", "cosine"])
+def test_update_w_prepare_query_accuracy(nn_data, metric):
+    nnd = NNDescent(
+        nn_data[200:800],
+        metric=metric,
+        n_neighbors=10,
+        random_state=None,
+        compressed=False,
+    )
+    nnd.prepare()
+
+    nnd.update(xs_fresh=nn_data[800:])
+    nnd.prepare()
+
+    knn_indices, _ = nnd.query(nn_data[:200], k=10, epsilon=0.2)
+
+    true_nnd = NearestNeighbors(metric=metric).fit(nn_data[200:])
+    true_indices = true_nnd.kneighbors(nn_data[:200], 10, return_distance=False)
+
+    num_correct = 0.0
+    for i in range(true_indices.shape[0]):
+        num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
+
+    percent_correct = num_correct / (true_indices.shape[0] * 10)
+    assert percent_correct >= 0.95, (
+        "NN-descent query did not get 95% " "accuracy on nearest neighbors"
+    )
+
+
+def evaluate_predictions(neighbors_true, neigbhors_computed, n_neighbors):
+    n_correct = 0
+    n_all = neighbors_true.shape[0] * n_neighbors
+    for i in range(neighbors_true.shape[0]):
+        n_correct += np.sum(np.in1d(neighbors_true[i], neigbhors_computed[i]))
+    return n_correct / n_all
+
+
+@pytest.mark.parametrize("metric", ["manhattan", "euclidean", "cosine"])
+@pytest.mark.parametrize("case", list(range(8)))  # the number of cases in update_data
+def test_update_with_changed_data(update_data, case, metric):
+    def evaluate(nn_descent, xs_to_fit, xs_to_query):
+        true_nn = NearestNeighbors(metric=metric, n_neighbors=k).fit(xs_to_fit)
+        neighbors, _ = nn_descent.query(xs_to_query, k=k)
+        neighbors_expected = true_nn.kneighbors(xs_to_query, k, return_distance=False)
+        p_correct = evaluate_predictions(neighbors_expected, neighbors, k)
+        assert p_correct >= 0.95, (
+            "NN-descent query did not get 95% " "accuracy on nearest neighbors"
+        )
+
+    k = 10
+    xs_orig, xs_fresh, xs_updated, indices_updated = update_data[case]
+    queries1 = xs_orig
+
+    # original
+    index = NNDescent(xs_orig, metric=metric, n_neighbors=40, random_state=1234)
+    index.prepare()
+    evaluate(index, xs_orig, queries1)
+    # updated
+    index.update(
+        xs_fresh=xs_fresh, xs_updated=xs_updated, updated_indices=indices_updated
+    )
+    if xs_fresh is not None:
+        xs = np.vstack((xs_orig, xs_fresh))
+        queries2 = np.vstack((queries1, xs_fresh))
+    else:
+        xs = xs_orig
+        queries2 = queries1
+    if indices_updated is not None:
+        xs[indices_updated] = xs_updated
+    evaluate(index, xs, queries2)
+    if indices_updated is not None:
+        evaluate(index, xs, xs_updated)
+
+
+@pytest.mark.parametrize("n_trees", [1, 2, 3, 10])
+def test_tree_numbers_after_multiple_updates(n_trees):
+    trees_after_update = max(1, int(np.round(n_trees / 3)))
+
+    nnd = NNDescent(np.array([[1.0]]), n_neighbors=1, n_trees=n_trees)
+
+    assert nnd.n_trees == n_trees, "NN-descent update changed the number of trees"
+    assert (
+        nnd.n_trees_after_update == trees_after_update
+    ), "The value of the n_trees_after_update in NN-descent after update(s) is wrong"
+    for i in range(5):
+        nnd.update(xs_fresh=np.array([[i]], dtype=np.float64))
+        assert (
+            nnd.n_trees == trees_after_update
+        ), "The value of the n_trees in NN-descent after update(s) is wrong"
+        assert (
+            nnd.n_trees_after_update == trees_after_update
+        ), "The value of the n_trees_after_update in NN-descent after update(s) is wrong"
+
+
 @pytest.mark.parametrize("metric", ["euclidean", "cosine"])
 def test_tree_init_false(nn_data, metric):
     nnd = NNDescent(
@@ -497,3 +600,66 @@ def test_tree_init_false(nn_data, metric):
     assert percent_correct >= 0.95, (
         "NN-descent query did not get 95% " "accuracy on nearest neighbors"
     )
+
+
+@pytest.mark.parametrize(
+    "metric", ["euclidean", "manhattan"]
+)  # cosine makes no sense for 1D
+def test_one_dimensional_data(nn_data, metric):
+    nnd = NNDescent(
+        nn_data[200:, :1],
+        metric=metric,
+        n_neighbors=20,
+        random_state=None,
+        tree_init=False,
+    )
+    nnd.prepare()
+
+    knn_indices, _ = nnd.query(nn_data[:200, :1], k=10, epsilon=0.2)
+
+    true_nnd = NearestNeighbors(metric=metric).fit(nn_data[200:, :1])
+    true_indices = true_nnd.kneighbors(nn_data[:200, :1], 10, return_distance=False)
+
+    num_correct = 0.0
+    for i in range(true_indices.shape[0]):
+        num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
+
+    percent_correct = num_correct / (true_indices.shape[0] * 10)
+    assert percent_correct >= 0.95, (
+        "NN-descent query did not get 95% " "accuracy on nearest neighbors"
+    )
+
+
+@pytest.mark.parametrize("metric", ["euclidean", "cosine"])
+def test_tree_no_split(small_data, sparse_small_data, metric):
+    k = 10
+    for data, data_type in zip([small_data, sparse_small_data], ["dense", "sparse"]):
+        n_instances = data.shape[0]
+        leaf_size = n_instances + 1  # just to be safe
+        data_train = data[n_instances // 2 :]
+        data_test = data[: n_instances // 2]
+
+        nnd = NNDescent(
+            data_train,
+            metric=metric,
+            n_neighbors=data_train.shape[0] - 1,
+            random_state=None,
+            tree_init=True,
+            leaf_size=leaf_size,
+        )
+        nnd.prepare()
+        knn_indices, _ = nnd.query(data_test, k=k, epsilon=0.2)
+
+        true_nnd = NearestNeighbors(metric=metric).fit(data_train)
+        true_indices = true_nnd.kneighbors(data_test, k, return_distance=False)
+
+        num_correct = 0.0
+        for i in range(true_indices.shape[0]):
+            num_correct += np.sum(np.in1d(true_indices[i], knn_indices[i]))
+
+        percent_correct = num_correct / (true_indices.shape[0] * k)
+        assert (
+            percent_correct >= 0.95
+        ), "NN-descent query did not get 95% for accuracy on nearest neighbors on {} data".format(
+            data_type
+        )
